package data;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import tools.ValueAndVector;

import java.util.Vector;



/** Class implementing a dimension holding values created by the projection of the data items
 *  on a principal component created by the PCA (Principal component analysis).
 */

public class PCADataDimension extends FloatDataDimension {

  private Vector<String> m_pDimensionNames; //vector holding the names of the dimensions from which this PCA Channel is created.
  private ValueAndVector  m_pPrincipalComponent; //eigenvector and eigenvalue of the principal component
  private int m_iComponentId;                   //ID of the principal component


  //////////////////////////////////////////
  // CONSTRUCTORS                         //
  //////////////////////////////////////////

  /** Creates a data dimension holding values created by the projection of the data items
   *  on a principal component created by the PCA (Principal component analysis).
   *
   *  @param pValues float array holding the projection values of the data items.
   *  @param pName String representing the name of the dimension.
   *  @param pDimensionNames a Vector holding Strings representing the dimension names of the dimensions used for the PCA.
   *  @param pPrincipalComp ValueAndVector object holding the eigenvalues and eigenvectors of the principal component.
   *  @param iCompId Integer indicating on which principal component the data items are mapped in this dimension.
   */
  public PCADataDimension(  float[] pValues,
                            String pName,
                            Vector<String> pDimensionNames,
                            ValueAndVector pPrincipalComp,
                            int iCompId) {

    super(pValues, pName);

    m_pDimensionNames = pDimensionNames;
    m_pPrincipalComponent = pPrincipalComp;
    m_iComponentId = iCompId;



  }



}