package data;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import listeners.ISelectionListener;

import java.util.Vector;

/** 
 * Abstract class implementing and specifying methods 
 * that each kind of selection should contain.
 */
public abstract class Selection {


  /* member variable */
  private Vector m_pSelectionListeners = null;
  private int m_iViewId = -1;




  /** Returns the ID of the view in which the selection was created.
   *
   *  @return ID of the view in which the selection was created.
   */
  public int getViewId() {
    return m_iViewId;
  }
  
  //////////////////////////////////////
  // PROTECTED SET METHODS            //
  //////////////////////////////////////
  /** Sets the ID of the view in which the selection was created.
   *
   *  @param iViewId The ID of the view in which the selection was created.
   */
  protected void setViewId(int iViewId) {
    m_iViewId = iViewId;
  }
  

  //////////////////////////////////////////
  // SELECTION LISTENER METHODS           //
  //////////////////////////////////////////
  /** Adds a ISelectionListener object that is notified, if changes in the selection occur.
   *
   * @param pListener ISelectionListener object that has to do some tasks if the Selection changes.
   *
   */
  public void addSelectionListener(ISelectionListener pListener) {
    if (m_pSelectionListeners == null) m_pSelectionListeners = new Vector();
    m_pSelectionListeners.add(pListener);
  }

  /** Method that is called from within the Selection implementation to notify all registered
   *  listeners if changes have occured.
   *
   */
  protected void selectionChanged() {
    if (m_pSelectionListeners == null) return;
  
    for (int i = 0; i < m_pSelectionListeners.size(); i++) {
      ISelectionListener pListener = (ISelectionListener)m_pSelectionListeners.get(i);
      pListener.selectionChanged(this);
    }

  }


}