package listeners;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import data.Data;

/** Interface for all classes that have to be notified, if there are changes in the data.
 */
public interface IDataSetListener {

  /** Invoked when the values of a dimension are modified.
   *
   *  @param pDataset Data object representing the dataset.
   *  @param pDimName  String object holding the name of the dimension of which the data item values have changed.
   */
  public void dimensionValuesChanged(Data pDataset, String pDimName);



  /** Invoked when a dimension was added or deleted from the dataset.
   *
   *  @param dataset Data object representing the dataset.
   *  @param dimensionName  String object holding the name of the dimension which has been added or deleted.
   */
  public void dimensionNumberChanged(Data pDataset, String pDimName);


}