package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */


import tools.KMeans;

import listeners.GlobalConstants;

import data.Data;


import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.awt.*;
import javax.swing.border.TitledBorder;



public class ClusteringFrame extends JFrame 
  implements WindowListener, ActionListener, ChangeListener, ListSelectionListener, GlobalConstants {

  //member variables

  private Data m_pData;          //data


  /*** member variables for the algorithm properties ***/
  private JTextField m_pClusterNumberTextField; //TextField holding the number of clusters
  private JTextField m_pIterationNumTextField;  //TextField holding the maximum number of iterations
  private JTextField m_pUpdateTextField;        //TextField holding the minimum update distance


  /*** member variables for the dimension weighting ***/
  private MyCellRenderer  m_pDimWeightCellRenderer;    //MyCellRenderer
  private JSlider         m_pDimWeightSlider;    //Slider
  private JList           m_pDimensionsList;   //List with the DimensionNames
  private JTextField      m_pDimWeightTextField;    //TextField holding the current weight

  /*** member variables: CLUSTERING **/
  private KMeans          m_pKMeans;         //object with the k means functionality




  /////////////////////////////////////
  // CONSTRUCTOR                     //
  /////////////////////////////////////
  //Constructor, Creating of GUI
  public ClusteringFrame(Data pData) {

    //1. Window-Properties
    setResizable(true);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    addWindowListener(this);
    setSize(new Dimension(400, 500));
    setTitle("K means clustering properties");

    //2. set the layout
    Container pContainer = getContentPane();
    pContainer.setLayout(new BoxLayout(pContainer, BoxLayout.Y_AXIS));


    //3. set member variables
    m_pData = pData;

    //4. create GUI components
    JPanel pAlgPropsPanelPanel = createAlgorithmPropsPanel();
    JPanel pDimWeightsPanel = createDimWeightsPanel();

    JButton pClusterButton = new JButton("Cluster");
    pClusterButton.setAlignmentX(CENTER_ALIGNMENT);
    pClusterButton.addActionListener(this);



    //5. Add Elements to Container
    pContainer.add(pAlgPropsPanelPanel);
    pContainer.add(pDimWeightsPanel);
    pContainer.add(new JLabel(" "));
    pContainer.add(pClusterButton);
    pContainer.add(new JLabel(" "));




  }


  ////////////////////////////////////
  // GUI CREATION METHODS           //
  ////////////////////////////////////

  private JPanel createAlgorithmPropsPanel() {

    //1. create the panel and the layout
    JPanel pAlgPropsPanel = new JPanel();
    GridBagLayout pGridBagLayout = new GridBagLayout();
    GridBagConstraints pConstraints = new GridBagConstraints();
    pAlgPropsPanel.setLayout(pGridBagLayout);


    //2. create the border
    TitledBorder pAlgPropsPanelBorder = new TitledBorder("Algorithm properties");
    pAlgPropsPanel.setBorder(pAlgPropsPanelBorder);


    //3. number of clusters
    JLabel pClusterNumLabel = new JLabel("Number of clusters (k): ");

    m_pClusterNumberTextField = new JTextField("4");
    m_pClusterNumberTextField.setHorizontalAlignment(JTextField.RIGHT);


    //4. number of maximal iteration
    JLabel pIterationNumLabel = new JLabel("Maximal iterations: ");

    m_pIterationNumTextField = new JTextField("10");
    m_pIterationNumTextField.setHorizontalAlignment(JTextField.RIGHT);


    //5. minimal update percentage
    JLabel pUpdateLabel = new JLabel("Minimal update distance (in % of range): ");

    m_pUpdateTextField = new JTextField("1");
    m_pUpdateTextField.setHorizontalAlignment(JTextField.RIGHT);



    //6. add the components
    pConstraints.fill = GridBagConstraints.BOTH;
    pConstraints.weightx = 1.0;
    pConstraints.ipadx = 50;
    pConstraints.insets = new Insets(5, 5, 5, 5);

    pGridBagLayout.setConstraints(pClusterNumLabel, pConstraints);
    pAlgPropsPanel.add(pClusterNumLabel);

    pConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagLayout.setConstraints(m_pClusterNumberTextField, pConstraints);
    pAlgPropsPanel.add(m_pClusterNumberTextField);


    pConstraints.gridwidth = 1; //set to default
    pGridBagLayout.setConstraints(pIterationNumLabel, pConstraints);
    pAlgPropsPanel.add(pIterationNumLabel);

    pConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagLayout.setConstraints(m_pIterationNumTextField, pConstraints);
    pAlgPropsPanel.add(m_pIterationNumTextField);

    pConstraints.gridwidth = 1; //set to default
    pGridBagLayout.setConstraints(pUpdateLabel, pConstraints);
    pAlgPropsPanel.add(pUpdateLabel);

    pConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagLayout.setConstraints(m_pUpdateTextField, pConstraints);
    pAlgPropsPanel.add(m_pUpdateTextField);



    //7. return panel
    return pAlgPropsPanel;

  }



  private JPanel createDimWeightsPanel() {

    //1. create the panel and the layout
    JPanel pDimWeightsPanel = new JPanel();
    pDimWeightsPanel.setLayout(new BoxLayout(pDimWeightsPanel, BoxLayout.Y_AXIS));

    //2. create the border
    TitledBorder pDimWeightsBorder = new TitledBorder("Dimension weighting");
    pDimWeightsPanel.setBorder(pDimWeightsBorder);


    //3. createSubPanel2
    JPanel pSubPanel1 = createDimWeightSubPanel1();
    pSubPanel1.setAlignmentX(CENTER_ALIGNMENT);
    JPanel pSubPanel2 = createDimWeightSubPanel2();


    //3. add the components
    pDimWeightsPanel.add(pSubPanel1);
    pDimWeightsPanel.add(new JLabel("          "));
    pDimWeightsPanel.add(pSubPanel2);

    //4. return panel
    return pDimWeightsPanel;


  }


  private JPanel createDimWeightSubPanel1() {

    //1. create the panel and the layout
    JPanel pSubPanel1 = new JPanel();
    pSubPanel1.setLayout(new BoxLayout(pSubPanel1, BoxLayout.X_AXIS));

    //2. Create a JList that displays the dimensions
    String[] pDimNames = m_pData.getDimensionNames();

    //delete DOI from list if the list contains it
    int iDimNum = pDimNames.length;
    for (int i = 0; i < iDimNum; i++) {
      if (pDimNames[i].equals(DOINAME)) {
        String[] pNewDimNames = new String[iDimNum-1];
        int addcount = 0;
        for (int j = 0; j < iDimNum-1; j++) {
          if (j == i) {
            addcount = 1;
          }
          pNewDimNames[j] = pDimNames[j+addcount];
        }
        pDimNames = pNewDimNames;
        iDimNum = pDimNames.length;
        break;
      }

    }
    m_pDimensionsList = new JList(pDimNames);
    m_pDimensionsList.addListSelectionListener(this);

    m_pDimWeightCellRenderer = new MyCellRenderer(iDimNum);

    m_pDimWeightCellRenderer.setAllWeightsToOne();
    m_pDimensionsList.setCellRenderer(m_pDimWeightCellRenderer);

    JScrollPane pScrollPane = new JScrollPane(m_pDimensionsList);
    pScrollPane.setPreferredSize(new Dimension(200, 100));
    pScrollPane.setMinimumSize(new Dimension(200, 100));



    //3. create JSlider to steer the weights of the dimensions
    m_pDimWeightSlider = new JSlider(JSlider.VERTICAL, 0, 255, 255);
    m_pDimWeightSlider.addChangeListener(this);

    //4. create subpanel
    JPanel pSubPanel3 = createDimWeightSubPanel3();
    pSubPanel3.setPreferredSize(new Dimension(40, 100));
    pSubPanel3.setMinimumSize(new Dimension(40, 100));
    pSubPanel3.setMaximumSize(new Dimension(40, 10000));


    //5. add components to the panel
    pSubPanel1.add(pScrollPane);
    pSubPanel1.add(new JLabel("       "));
    pSubPanel1.add(m_pDimWeightSlider);
    pSubPanel1.add(pSubPanel3);

    //6. return the panel
    return pSubPanel1;

  }


  private JPanel createDimWeightSubPanel2() {
    //1. create the panel and the layout
    JPanel pSubPanel2 = new JPanel();
    pSubPanel2.setLayout(new BoxLayout(pSubPanel2, BoxLayout.X_AXIS));

    //2. buttons for setting all values to 1 or 0
    JButton pZeroButton = new JButton("Set all to 0");
    pZeroButton.addActionListener(this);
    JButton pOneButton = new JButton("Set all to 1");
    pOneButton.addActionListener(this);

    //add components to the panel
    pSubPanel2.add(pZeroButton);
    pSubPanel2.add(new JLabel("          "));
    pSubPanel2.add(pOneButton);

    //return the panel
    return pSubPanel2;

  }


  private JPanel createDimWeightSubPanel3() {
    //1. create the panel and the layout
    JPanel pSubPanel3 = new JPanel();
    pSubPanel3.setLayout(new BoxLayout(pSubPanel3, BoxLayout.Y_AXIS));
    //pSubPanel3.setLayout(new BorderLayout());

    //2. create the labels
    JLabel pOneLabel = new JLabel("1");
    pOneLabel.setHorizontalAlignment(JLabel.LEFT);
    pOneLabel.setVerticalAlignment(JLabel.TOP);
    pOneLabel.setPreferredSize(new Dimension(40, 20));
    pOneLabel.setMaximumSize(new Dimension(40, 20));
    pOneLabel.setAlignmentX(LEFT_ALIGNMENT);


    JLabel pZeroLabel = new JLabel("0");
    pZeroLabel.setHorizontalAlignment(JLabel.LEFT);
    pZeroLabel.setVerticalAlignment(JLabel.BOTTOM);
    pZeroLabel.setPreferredSize(new Dimension(40, 20));
    pZeroLabel.setMaximumSize(new Dimension(40, 20));
    pZeroLabel.setAlignmentX(LEFT_ALIGNMENT);


    //3. create the textfield
    m_pDimWeightTextField = new JTextField("1");
    m_pDimWeightTextField.setHorizontalAlignment(JTextField.RIGHT);
    m_pDimWeightTextField.setPreferredSize(new Dimension(40, 20));
    m_pDimWeightTextField.setMaximumSize(new Dimension(40, 20));
    m_pDimWeightTextField.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
       dimWeightTextField_actionPerformed(e);
      }
    });


    //4. add components to the panel
    pSubPanel3.add(pOneLabel);
    pSubPanel3.add(Box.createGlue());
    pSubPanel3.add(m_pDimWeightTextField);
    pSubPanel3.add(Box.createGlue());
    pSubPanel3.add(pZeroLabel);
    

    //5. return the panel
    return pSubPanel3;

  }


  ////////////////////////////////////
  // ACTIONLISTENER                 //
  ////////////////////////////////////
  public void actionPerformed(ActionEvent e) {

    //1. dimension weighting: Set all to 0 - Button
    if (e.getActionCommand().equals("Set all to 0")) {
      m_pDimWeightCellRenderer.setAllWeightsToZero();
      m_pDimWeightSlider.setValue(m_pDimWeightSlider.getMinimum());

      m_pDimensionsList.revalidate();
      m_pDimensionsList.repaint();
      return;
    }

    //2. imension weighting: Set all to 1 - Button
    if (e.getActionCommand().equals("Set all to 1")) {
      m_pDimWeightCellRenderer.setAllWeightsToOne();
      m_pDimWeightSlider.setValue(m_pDimWeightSlider.getMaximum());

      m_pDimensionsList.revalidate();
      m_pDimensionsList.repaint();
      return;
    }


    //3. cluster button: checks all inputs and starts clustering
    if (e.getActionCommand().equals("Cluster")) {

      //3.1. number of clusters (k)
      int iClusterNum = -1;
      try {
        iClusterNum = Integer.parseInt(m_pClusterNumberTextField.getText());
      }
      catch (Exception ex) {
        iClusterNum = -1;
      }

      if (iClusterNum < 2) {
        String pMessage = "The number of clusters must be an integer greater than 1";
        String pTitle = "Wrong number of clusters";
        JOptionPane.showMessageDialog(this, pMessage, pTitle, JOptionPane.ERROR_MESSAGE);
        return;
      }


      //3.2. number of iterations
      int iIterationNum = -1;
      try {
        iIterationNum = Integer.parseInt(m_pIterationNumTextField.getText());
      }
      catch (Exception ex) {
        iIterationNum = -1;
      }

      if (iIterationNum < 1) {
        String pMessage = "The number of iterations must be an integer greater than 0";
        String pTitle = "Wrong number of iterations";
        JOptionPane.showMessageDialog(this, pMessage, pTitle, JOptionPane.ERROR_MESSAGE);
        return;
      }


      //3.3. update distance (in % of range)
      int iUpdateDistance = -1;
      try {
        iUpdateDistance = Integer.parseInt(m_pUpdateTextField.getText());
      }
      catch (Exception ex) {
        iUpdateDistance = -1;
      }

      if ((iUpdateDistance < 0) || (iUpdateDistance > 50)) {
        String pMessage = "The minimum update distance must be an integer between 0 and 50.\n\n";
        pMessage += "This parameter has to be given in percent of the range of the data. \n";
        pMessage += "The range is the maximal value minus the minimum value";
        String pTitle = "Wrong minimum update distance";
        JOptionPane.showMessageDialog(this, pMessage, pTitle, JOptionPane.ERROR_MESSAGE);
        return;
      }


      //3.4. dimension weights

      //create an array for the normalized weights for each dimension of the data set
      int[] pWeights = m_pDimWeightCellRenderer.getWeights();
      int iWeightNum = pWeights.length;
      
      int iDimNum = m_pData.getDimNumber();
      float[] pNormWeights = new float[iDimNum];
      for (int i = 0; i < iDimNum; i++) {
        pNormWeights[i] = 0.0f;
      }


      //normalize and check if there is at least one weight > 0
      boolean bZeroWeights = true;
      String[] pDimNames = m_pData.getDimensionNames();
      int iDimNamesNum = pDimNames.length;
      for (int i = 0; i < iWeightNum; i++) {

        //calculate the dimID
        int iDimId = -1;
        String pCurrDimName = (String)m_pDimensionsList.getModel().getElementAt(i);
        for (int j = 0; j < iDimNamesNum; j++) {
          if (pDimNames[j].equals(pCurrDimName)) {
            iDimId = j;
            break;
          }
        }

        if (iDimId < 0) continue;

        //set and check the weight values
        pNormWeights[iDimId] = (float)pWeights[i]/255.0f;

        if (pWeights[i] > 0) {
          bZeroWeights = false;
        }
      }

      if (bZeroWeights) {
        String pMessage = "At least one dimension must have a weight greater than 0.";
        String pTitle = "All dimension weights are zero";
        JOptionPane.showMessageDialog(this, pMessage, pTitle, JOptionPane.ERROR_MESSAGE);
        return;
      }

      //TO DO: fix errors
      m_pKMeans = new KMeans(m_pData, iClusterNum, iIterationNum, iUpdateDistance, pNormWeights);

      String pTitle = "Data clustered.";
      String pMessage = "The dataset has been clustered successfully.";
      JOptionPane.showMessageDialog(this, pMessage, pTitle, JOptionPane.INFORMATION_MESSAGE);


      return;



    }

  }

  //ActionListener method for the Textfield holding the current dimension value
  private void dimWeightTextField_actionPerformed(ActionEvent e) {
    try {
      float fValue = Float.parseFloat(m_pDimWeightTextField.getText());
      if (fValue < 0) {
        fValue = 0.0f;
        m_pDimWeightTextField.setText("0.00");
      }
      if (fValue > 1) {
        fValue = 1.0f;
        m_pDimWeightTextField.setText("1.00");
      }

      m_pDimWeightSlider.setValue((int)(fValue*255));

    }
    catch (Exception ex) {
      int iValue = m_pDimWeightSlider.getValue();
      double dTextValue = (double)((int)((double)(iValue*100)/(double)m_pDimWeightSlider.getMaximum()));
      dTextValue /= 100;
      m_pDimWeightTextField.setText(dTextValue+"");

    }
  }

  ////////////////////////////////////
  // CHANGELISTENER                 //
  ////////////////////////////////////
  public void stateChanged(ChangeEvent e) {

    JSlider pSlider = (JSlider)e.getSource();
    int iValue = pSlider.getValue();

    //set the weights in the cellrenderer
    int[] pSelectedIndices = m_pDimensionsList.getSelectedIndices();
    int iIndicesNum = pSelectedIndices.length;
    for (int i = 0; i < iIndicesNum; i++) {
      m_pDimWeightCellRenderer.setWeight(pSelectedIndices[i], iValue);
    }
    m_pDimensionsList.revalidate();
    m_pDimensionsList.repaint();


    //set the weight in the current weight textfield
    double dTextValue = (double)((int)((double)(iValue*100)/(double)pSlider.getMaximum()));
    dTextValue /= 100;
    m_pDimWeightTextField.setText(dTextValue+"");


  }

  ////////////////////////////////////
  // LISTSELECTIONLISTENER          //
  ////////////////////////////////////
  public void valueChanged(ListSelectionEvent e) {
    int iSelectedIndex = m_pDimensionsList.getSelectedIndex();
    int iValue = m_pDimWeightCellRenderer.getWeight(iSelectedIndex);

    m_pDimWeightSlider.setValue(iValue);

    //set the weight in the current weight textfield
    double dTextValue = (double)((int)((double)(iValue*100)/(double)m_pDimWeightSlider.getMaximum()));
    dTextValue /= 100;
    m_pDimWeightTextField.setText(dTextValue+"");


  }


  ////////////////////////////////////
  // WINDOWLISTENER                 //
  ////////////////////////////////////
  public void windowClosing(WindowEvent event1) {
    setVisible(false);
  }
  public void windowGainedFocus(WindowEvent event2) {}
  public void windowClosed(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}


}