package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import listeners.IExportable;

import javax.swing.JFrame;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.File;
import javax.imageio.ImageIO;



//class implementing the export of an image
public class ImageExportFrame extends JFrame implements WindowListener, ActionListener {


  //member variables
  private IExportable m_pExportable;   //object from which an image should be exported
  private JFrame      m_pOwnerFrame;            //jframe from which this frame is opened (needed to disable the owner)
  private File        m_pOutputFile;       //file into which the image is written.

  //GUI elements
  JRadioButton  m_pCurrentSizeRadioButton, m_pUserDefinedSizeRadioButton;
  JTextField    m_pWidthTextfield, m_pHeightTextfield;






  /** constructor **/
  public ImageExportFrame(IExportable pExportable, JFrame pOwnerFrame) {

    //1. set Window-Properties
    setResizable(true);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    setSize(new Dimension(290, 180));
    setTitle("Image export");


    //2. add Listener functionality
    addWindowListener(this);


    //3. layout and container
    Container pContainer = getContentPane();
    pContainer.setLayout(null);


    //4. set member variables
    m_pExportable = pExportable;
    m_pOwnerFrame = pOwnerFrame;
    m_pOutputFile = null;



    //5. CREATE THE GUI
    //TO DO: create separate panels
    //5.1. create radiobuttons
    m_pCurrentSizeRadioButton = new JRadioButton("Current size (" + m_pExportable.getWidth() + " x " + m_pExportable.getHeight() + ")");
    m_pCurrentSizeRadioButton.setSelected(true);
    m_pCurrentSizeRadioButton.setSize(200, 20);
    m_pCurrentSizeRadioButton.setLocation(20, 20);
    m_pCurrentSizeRadioButton.addActionListener(this);

    m_pUserDefinedSizeRadioButton = new JRadioButton("User defined size");
    m_pUserDefinedSizeRadioButton.setSize(200, 20);
    m_pUserDefinedSizeRadioButton.setLocation(20, 50);
    m_pUserDefinedSizeRadioButton.addActionListener(this);


    //5.2. add buttons to button group
    ButtonGroup pButtonGroupSize = new ButtonGroup();
    pButtonGroupSize.add(m_pCurrentSizeRadioButton);
    pButtonGroupSize.add(m_pUserDefinedSizeRadioButton);


    //5.3. create width input
    JLabel pWidthLabel = new JLabel("Width: ");
    pWidthLabel.setSize(50,20);
    pWidthLabel.setLocation(50,80);

    m_pWidthTextfield = new JTextField("800");
    m_pWidthTextfield.setHorizontalAlignment(JTextField.RIGHT);
    m_pWidthTextfield.setSize(50,20);
    m_pWidthTextfield.setLocation(100,80);
    m_pWidthTextfield.setEnabled(false);


    //5.4. create height input
    JLabel pHeightLabel = new JLabel("Height:");
    pHeightLabel.setSize(50,20);
    pHeightLabel.setLocation(170,80);

    m_pHeightTextfield = new JTextField("600");
    m_pHeightTextfield.setHorizontalAlignment(JTextField.RIGHT);
    m_pHeightTextfield.setSize(50,20);
    m_pHeightTextfield.setLocation(220,80);
    m_pHeightTextfield.setEnabled(false);


    //5.5. create buttons
    JButton pOKButton = new JButton("OK");
    pOKButton.setSize(100, 25);
    pOKButton.setLocation(20, 120);
    pOKButton.addActionListener(this);


    JButton pAbortButton = new JButton("Abort");
    pAbortButton.setSize(100, 25);
    pAbortButton.setLocation(170, 120);
    pAbortButton.addActionListener(this);




    //5.6. add gui components to the container
    pContainer.add(m_pCurrentSizeRadioButton);
    pContainer.add(m_pUserDefinedSizeRadioButton);

    pContainer.add(pWidthLabel);
    pContainer.add(m_pWidthTextfield);

    pContainer.add(pHeightLabel);
    pContainer.add(m_pHeightTextfield);

    pContainer.add(pOKButton);
    pContainer.add(pAbortButton);

  }



  ////////////////////////////////////////
  //  SET METHODS                       //
  ////////////////////////////////////////

  public void setExportable(IExportable pExportable) {
    //1. make an initial check
    if (pExportable == null) return;
    
    //2. set the exportable object and the corresponding text of the current size radio button
    m_pExportable = pExportable;
    m_pCurrentSizeRadioButton.setText("Current size (" + m_pExportable.getWidth() + " x " + m_pExportable.getHeight() + ")");
  }


  public void setOutputFile(File pFile) {
    m_pOutputFile = pFile;
  }


  ////////////////////////////////////////
  // ACTION LISTENER METHOD             //
  ////////////////////////////////////////
  public void actionPerformed(ActionEvent e) {
    //1. fetch the first character of the action command to decide, what is to do
    char cHelpChar = e.getActionCommand().charAt(0);
    
    //2. The selection of the radio buttons has changed.
    if ((cHelpChar == 'C') || (cHelpChar == 'U'))  {
      if (m_pCurrentSizeRadioButton.isSelected()) {
        m_pWidthTextfield.setEnabled(false);
        m_pHeightTextfield.setEnabled(false);
      }
      else {
        m_pWidthTextfield.setEnabled(true);
        m_pHeightTextfield.setEnabled(true);
      }
      return;

    }

    //3. make the export
    if (e.getActionCommand().equals("OK")) {

      //3.1. check which size should be used
      int iWidth, iHeight;
      if (m_pCurrentSizeRadioButton.isSelected()) {
        iWidth = m_pExportable.getWidth();
        iHeight = m_pExportable.getHeight();
      }
      else {
        try{
          iWidth = Integer.parseInt(m_pWidthTextfield.getText());
          iHeight = Integer.parseInt(m_pHeightTextfield.getText());
        }
        catch (Exception pException) {
          iWidth = -1;
          iHeight = -1;
        }
      }


      //3.2. check if size is valid
      if ((iWidth < 200) || (iHeight < 200)) {
        String pMessage = "The width and the height of the exported image have to be bigger than 199 Pixels.";
        String pTitle = "Invalid width or height";
        JOptionPane.showMessageDialog(this, pMessage, pTitle, JOptionPane.ERROR_MESSAGE);
        return;
      }

      //3.3. render the image
      BufferedImage pImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
      m_pExportable.renderImage(pImage);

      //3.4. write the image
      try {
        String pFileName = m_pOutputFile.getName();
        String pFormatName = "png";
        if (pFileName.length() > 4) {
          String pFileExtension = pFileName.substring(pFileName.length()-3);
          if (pFileExtension.equals("jpg")) {
            pFormatName = "jpg";
          }
          else if (pFileExtension.equals("gif")) {
            pFormatName = "gif";
          }
          else if (pFileExtension.equals("bmp")) {
            pFormatName = "bmp";
          }
        }
        
        ImageIO.write(pImage, pFormatName, m_pOutputFile);
       
      }
      catch (Exception pException) {}

    }

    //4. close the frame and set the owner active
    setVisible(false);
    if (m_pOwnerFrame != null) m_pOwnerFrame.setEnabled(true);
    return;


  }


  ////////////////////////////////////////
  //WINDOWS LISTENER METHODS            //
  ////////////////////////////////////////
  public void windowClosing(WindowEvent event1) {
    //close the frame and set the owner active
    setVisible(false);
    if (m_pOwnerFrame != null) m_pOwnerFrame.setEnabled(true);
  }
  public void windowGainedFocus(WindowEvent event2) {}
  public void windowClosed(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}


}