package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */


import javax.swing.JFrame;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.File;
import javax.imageio.ImageIO;



/**
 * This class creates and handles the functionality the interface of the program.
 * It also represents the window in which the parallel coordinates view is shown and all functionalities
 * are called via the menus.
 *
 * @author    J&uuml;rgen Platzer
 */
//class creates and handles the functionality the interface of the application
public class OutlierDetectionFrame extends JFrame implements WindowListener, ChangeListener, KeyListener{

  //member variables

  private OutlierDetectionView m_pOutlierDetectionView;    //outlier detection view

  private JSlider m_pDecisionSlider;   //slider that decides what are outliers
  private JLabel  m_pDecisionLabel;    //label that shows how much percent of outliers are expected


  //Constructor, Creating of GUI

  /**
   * Constructor that creates the main window and objects of all other windows and views.
   * The needed listeners are registered.
   *
   */

  public OutlierDetectionFrame(OutlierDetectionView outlierDetectionView) {

    //1. set Window-Properties
    setResizable(true);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    setTitle("Outlier Detection");
    setSize(new Dimension(805, 700));

    //2. add Listener functionality
    addWindowListener(this);
    addKeyListener(this);  //TO DO: check if this is really needed.


    //3. Containers-Properties
    Container pContainer = this.getContentPane();
    pContainer.setLayout(new BorderLayout());

    
    //4. create the panels 
    m_pOutlierDetectionView = outlierDetectionView;
    JPanel pOutlierPanel = createOutlierPanel();

    //5. add the components
    pContainer.add(m_pOutlierDetectionView, BorderLayout.CENTER);
    pContainer.add(pOutlierPanel, BorderLayout.EAST);




  }


  ////////////////////////////////////////
  //GUI CREATION METHODS                //
  ////////////////////////////////////////
  
  
  private JPanel createOutlierPanel() {
    //1. create the panel and set the layout
    JPanel pOutlierPanel = new JPanel();
    pOutlierPanel.setLayout(new BoxLayout(pOutlierPanel, BoxLayout.Y_AXIS));

    //2. decision label
    m_pDecisionLabel = new JLabel("97.5 % non-outliers expected");
    m_pDecisionLabel.setHorizontalAlignment(JLabel.CENTER);

    //3. decision slider
    m_pDecisionSlider = new JSlider(JSlider.VERTICAL, 0, 1000, 975);
    m_pDecisionSlider.addChangeListener(this);

    //4. add the components and return the panel
    pOutlierPanel.add(m_pDecisionLabel);
    pOutlierPanel.add(m_pDecisionSlider);

    return pOutlierPanel;

  }


  ////////////////////////////////////////
  //SET METHODS                         //
  ////////////////////////////////////////
  public void setDistances(float[] pRobustDistances, float[] pMahalanobisDistances, int iDegreeOfFreedom) {
    m_pOutlierDetectionView.setDistances(pRobustDistances, pMahalanobisDistances, iDegreeOfFreedom);
  }

  ////////////////////////////////////////
  //ChangeListener method               //
  ////////////////////////////////////////

  /**
   * Handles the action that is initiated by choosing a MenuItem.
   *
   * @param e ActionEvent that identifies the MenuItem from which the event is triggered.
   */
  public void stateChanged(ChangeEvent e)  {

    int iValue = m_pDecisionSlider.getValue();

    float fValue = (float)iValue/10;
    m_pDecisionLabel.setText(fValue + " % outlier expected");


    m_pOutlierDetectionView.setOutlierPercentage(fValue/100);


  }









  ////////////////////////////////////////
  //KeyListener methods                 //
  ////////////////////////////////////////

  /**
   * Sets a flag in the views to true if a certain key is pressed.
   */
   public void keyPressed(KeyEvent e) {
     if (e.getKeyCode() == KeyEvent.VK_CONTROL) {
       //parcoordview.setControlPressed(true);
     }
   }

  /**
   * Sets all flags indicating a pressed key in the views to false.
   */
   public void keyReleased(KeyEvent e) {
     //parcoordview.setControlPressed(false);
   }

  /**
   * Unused KeyListener method.
   */
   public void keyTyped(KeyEvent e) {
   }





  ////////////////////////////////////////
  //WindowListener methods              //
  ////////////////////////////////////////
  /**
   * Closing the window.
   */
  public void windowClosing(WindowEvent event1) {
    setVisible(false);
  }

  /**
   * Unused WindowListener method.
   */
  public void windowGainedFocus(WindowEvent event2) {}

  /**
   * Unused WindowListener method.
   */
  public void windowClosed(WindowEvent e) {}
  /**
   * Unused WindowListener method.
   */
  public void windowDeactivated(WindowEvent e) {}
  /**
   * Unused WindowListener method.
   */
  public void windowActivated(WindowEvent e) {}
  /**
   * Unused WindowListener method.
   */
  public void windowDeiconified(WindowEvent e) {}
  /**
   * Unused WindowListener method.
   */
  public void windowIconified(WindowEvent e) {}
  /**
   * Unused WindowListener method.
   */
  public void windowOpened(WindowEvent e) {}








}