package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import data.Data;
import data.PCADataDimension;
import listeners.GlobalConstants;
import tools.PrincipalComponentAnalysis;
import tools.ValueAndVector;


import javax.swing.JFrame;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.TitledBorder;

import java.util.Vector;


public class PCAFrame extends JFrame implements WindowListener, ActionListener, ItemListener,
ChangeListener, ListSelectionListener, GlobalConstants {


  //member variables


  /*** GUI elements  ***/
  private JRadioButton m_pClassicRadioButton, m_pRobustRadioButton;   //radiobuttons that decide wether robust or classic pca
  private JSlider m_pRobustnessSlider;                          //slider for robustness parameter
  private JTextField m_pRobustnessTextField;                       //textfield for robustness parameter
  private JTextField m_pSaveNameTextField;                //textfield for the name of the principal components to save
  private JLabel m_pExplainedVarianceLabel;
  private Choice m_pUsedPCsChoice;


  /*** data elements ***/
  private Data      m_pData;           //data object
  private JList     m_pDimensionsList;   //list holding the relevant dimensions for PCA


  /*** PCA ***/
  private PrincipalComponentAnalysis m_pPCA;


 //Constructor, Creating of GUI
  public PCAFrame(Data pData) {

    //1. set the member variables
    m_pData = pData;
    m_pPCA = null;


    //2. set the Windows Properties
    setResizable(true);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    setSize(new Dimension(650, 500));
    setTitle("Principal Component Analysis");
    addWindowListener(this);

    
    //3. Container Properties
    Container pContainer = this.getContentPane();
    GridBagLayout pGridBagLayout = new GridBagLayout();
    GridBagConstraints pConstraints = new GridBagConstraints();
    pContainer.setLayout(pGridBagLayout);


    //4. create the panels
    JPanel pPCATypePanel = createPCATypePanel();
    JPanel pRobustParamsPanel = createRobustParamsPanel();
    JPanel pPCAParamsPanel = createPCAParamsPanel();
    JPanel pButtonsPanel = createButtonPanel();


    //5. add the panels to the container
    pConstraints.fill = GridBagConstraints.BOTH;
    pConstraints.weightx = 1.0;
    pConstraints.ipadx = 50;
    pConstraints.insets = new Insets(5, 5, 5, 5);

    pConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagLayout.setConstraints(pPCATypePanel, pConstraints);
    pContainer.add(pPCATypePanel);

    pGridBagLayout.setConstraints(pRobustParamsPanel, pConstraints);
    pContainer.add(pRobustParamsPanel);

    pGridBagLayout.setConstraints(pPCAParamsPanel, pConstraints);
    pContainer.add(pPCAParamsPanel);

    pGridBagLayout.setConstraints(pButtonsPanel, pConstraints);
    pContainer.add(pButtonsPanel);

  }

  ////////////////////////////////////
  // GUI CREATION METHODS           //
  ////////////////////////////////////

  //creates a panel for deciding which type of pca should be used (classic, robust)
  private JPanel createPCATypePanel() {


    //1. create the panel and the layout
    JPanel pPCATypePanel = new JPanel();
    pPCATypePanel.setLayout(new FlowLayout());


    //2. create the border
    TitledBorder pPCATypeBorder = new TitledBorder("Type of pca");
    pPCATypePanel.setBorder(pPCATypeBorder);



    //3. create the radiobuttons
    m_pClassicRadioButton = new JRadioButton("Classic PCA");
    m_pClassicRadioButton.setSelected(true);
    m_pClassicRadioButton.addActionListener(this);

    m_pRobustRadioButton = new JRadioButton("Robust PCA");
    m_pRobustRadioButton.addActionListener(this);


    //4. add the buttons to a button group
    ButtonGroup pTypeGroup = new ButtonGroup();
    pTypeGroup.add(m_pClassicRadioButton);
    pTypeGroup.add(m_pRobustRadioButton);


    //5. add the components to the panel
    pPCATypePanel.add(m_pClassicRadioButton);
    pPCATypePanel.add(m_pRobustRadioButton);

    return pPCATypePanel;


  }



  //creates a panel for the robust parameters
  private JPanel createRobustParamsPanel() {

    //1. create the panel and the layout
    JPanel pRobustParamsPanel = new JPanel();
    GridBagLayout pGridBagLayout = new GridBagLayout();
    GridBagConstraints pConstraints = new GridBagConstraints();
    pRobustParamsPanel.setLayout(pGridBagLayout);


    //2. create the border
    TitledBorder pRobustParamsBorder = new TitledBorder("Robustness parameter");
    pRobustParamsPanel.setBorder(pRobustParamsBorder);


    //ROBUST PARAMETER H (which percentage of the data)

    //3. labels
    JLabel pRobustnessLabel = new JLabel("Percentage of data (h) for robust estimate");
    JLabel pRobustnessMinLabel = new JLabel("50 %");
    JLabel pRobustnessMaxLabel = new JLabel("100 %");
    pRobustnessMaxLabel.setHorizontalAlignment(JLabel.RIGHT);


    //4. JSlider
    m_pRobustnessSlider = new JSlider(50, 100, 50);
    m_pRobustnessSlider.setEnabled(false);
    m_pRobustnessSlider.addChangeListener(this);

    //5. TextField
    m_pRobustnessTextField = new JTextField("50 %");
    m_pRobustnessTextField.setHorizontalAlignment(JTextField.RIGHT);
    m_pRobustnessTextField.setEnabled(false);
    m_pRobustnessTextField.addActionListener(this);
    m_pRobustnessTextField.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        robustnessTextField_actionPerformed(e);
      }
    });


    //6. add the components to the panel
    pConstraints.fill = GridBagConstraints.BOTH;
    pConstraints.weightx = 1.0;
    pConstraints.ipadx = 50;
    pConstraints.insets = new Insets(5, 5, 5, 5);

    pConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagLayout.setConstraints(pRobustnessLabel, pConstraints);
    pRobustParamsPanel.add(pRobustnessLabel);

    pConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagLayout.setConstraints(m_pRobustnessSlider, pConstraints);
    pRobustParamsPanel.add(m_pRobustnessSlider);

    pConstraints.gridwidth = 1;
    pConstraints.weightx = 1;
    pGridBagLayout.setConstraints(pRobustnessMinLabel, pConstraints);
    pRobustParamsPanel.add(pRobustnessMinLabel);

    pConstraints.weightx = 0;
    pGridBagLayout.setConstraints(m_pRobustnessTextField, pConstraints);
    pRobustParamsPanel.add(m_pRobustnessTextField);


    pConstraints.weightx = 1;
    pConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagLayout.setConstraints(pRobustnessMaxLabel, pConstraints);
    pRobustParamsPanel.add(pRobustnessMaxLabel);

    return pRobustParamsPanel;

  }


  //creates a panel for the PCA parameters
  private JPanel createPCAParamsPanel() {

    //1. create the panel and the layout
    JPanel pPCAParamsPanel = new JPanel();
    pPCAParamsPanel.setLayout(new BoxLayout(pPCAParamsPanel, BoxLayout.X_AXIS));

    //2. create the border
    TitledBorder pPCAParamsBorder = new TitledBorder("PCA parameters");
    pPCAParamsPanel.setBorder(pPCAParamsBorder);


    //3. Create a JList that displays the dimensions
    //   TO DO: This loop is often applied -> it should be provided by the data object
    String[] pDimNames = m_pData.getDimensionNames();

    //delete DOI from list if the list contains it
    int iDimNum = pDimNames.length;
    for (int i = 0; i < iDimNum; i++) {
      if (pDimNames[i].equals(DOINAME)) {
        String[] pNewDimNames = new String[iDimNum-1];
        int addcount = 0;
        for (int j = 0; j < iDimNum-1; j++) {
          if (j == i) {
            addcount = 1;
          }
          pNewDimNames[j] = pDimNames[j+addcount];
        }
        pDimNames = pNewDimNames;
        iDimNum--;
        break;
      }

    }
    m_pDimensionsList = new JList(pDimNames);
    m_pDimensionsList.addListSelectionListener(this);

    JScrollPane pScrollPane = new JScrollPane(m_pDimensionsList);
    pScrollPane.setPreferredSize(new Dimension(200, 100));
    pScrollPane.setMinimumSize(new Dimension(200, 100));
    pScrollPane.setMaximumSize(new Dimension(200, 100));



    //4. create subpanel
    JPanel pSubPanel = createPCAParamsSubPanel();
    pSubPanel.setPreferredSize(new Dimension(40, 100));
    pSubPanel.setMinimumSize(new Dimension(40, 100));


    //5. add components to the panel and return the panel
    pPCAParamsPanel.add(pScrollPane);
    pPCAParamsPanel.add(new JLabel("       "));
    pPCAParamsPanel.add(pSubPanel);

    return pPCAParamsPanel;

  }

  
  //creates a subpanel containing number of PCs and explained variance for the PCAParamsPanel
  private JPanel createPCAParamsSubPanel() {


    //1. create the panel and the layout
    JPanel pPCAParamsSubPanel = new JPanel();
    GridBagLayout pGridBagLayout = new GridBagLayout();
    GridBagConstraints pConstraints = new GridBagConstraints();
    pPCAParamsSubPanel.setLayout(pGridBagLayout);


    //2. Labels
    JLabel pPCNumLabel = new JLabel("Number of principal components: ");
    m_pExplainedVarianceLabel = new JLabel("Explained Variance: NA");
    JLabel pSaveNameLabel = new JLabel("Name for PCA channels: ");

    //3. Choice
    m_pUsedPCsChoice = new Choice();
    m_pUsedPCsChoice.addItemListener(this);

    //4. TextField
    m_pSaveNameTextField = new JTextField("Principal component");


    //add the elements to the panel
    pConstraints.fill = GridBagConstraints.BOTH;
    pConstraints.weightx = 1.0;
    pConstraints.ipadx = 50;
    pConstraints.insets = new Insets(5, 5, 5, 5);

    pConstraints.gridwidth = 1;
    pGridBagLayout.setConstraints(pPCNumLabel, pConstraints);
    pPCAParamsSubPanel.add(pPCNumLabel);

    pConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagLayout.setConstraints(m_pUsedPCsChoice, pConstraints);
    pPCAParamsSubPanel.add(m_pUsedPCsChoice);

    pGridBagLayout.setConstraints(m_pExplainedVarianceLabel, pConstraints);
    pPCAParamsSubPanel.add(m_pExplainedVarianceLabel);

    pConstraints.gridwidth = 1;
    pConstraints.weightx = 0;
    pGridBagLayout.setConstraints(pSaveNameLabel, pConstraints);
    pPCAParamsSubPanel.add(pSaveNameLabel);

    pConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pConstraints.weightx = 1.0;
    pGridBagLayout.setConstraints(m_pSaveNameTextField, pConstraints);
    pPCAParamsSubPanel.add(m_pSaveNameTextField);

    return pPCAParamsSubPanel;
  }


  //creates the panel containing all Buttons
  private JPanel createButtonPanel() {

    //1. create the panel and the layout
    JPanel pButtonsPanel = new JPanel();
    pButtonsPanel.setLayout(new FlowLayout());

    //2. buttons
    JButton pSavePCAButton = new JButton("Add principal components to data");
    pSavePCAButton.addActionListener(this);

    //3. add components to panel and return it
    pButtonsPanel.add(pSavePCAButton);

    return pButtonsPanel;

  }




  ////////////////////////////////////////
  //ACTIONLISTENER methods              //
  ////////////////////////////////////////
  public void actionPerformed(ActionEvent e) {
    //System.out.println(e.getActionCommand());

    //1. check if robust or classic pca is active
    if ((e.getActionCommand().equals("Robust PCA")) || (e.getActionCommand().equals("Classic PCA")))  {

      //1.1. the robust PCA should be calculated
      if (m_pRobustRadioButton.isSelected()) {
        m_pRobustnessSlider.setEnabled(true);
        m_pRobustnessTextField.setEnabled(true);

        float fRobustness = (float)m_pRobustnessSlider.getValue()/100;
        if ((m_pPCA != null) && (m_pPCA.getRobustnessParameter() != fRobustness)) {
          calculatePCA(true);
        }
      }
      //1.2. the classic PCA should be calculated
      else {
        m_pRobustnessSlider.setEnabled(false);
        m_pRobustnessTextField.setEnabled(false);

        if ((m_pPCA != null) && (m_pPCA.getRobustnessParameter() != 1)) {
          calculatePCA(true);
        }

      }

      return;
    }

    //2. save the PCA result to the data
    if (e.getActionCommand().equals("Add principal components to data")) {
      savePCAtoData();
      return;
    }


  }

  //action of changing the value in the robustness textfield
  private void robustnessTextField_actionPerformed(ActionEvent e) {
    try {
      int iValue = Integer.parseInt(m_pRobustnessTextField.getText());

      if (iValue < 50) {
        iValue = 50;
        m_pRobustnessTextField.setText("50 %");
      }

      if (iValue > 100) {
        iValue = 100;
        m_pRobustnessTextField.setText("100 %");
      }

      m_pRobustnessSlider.setValue(iValue);

      float fRobustness = (float)m_pRobustnessSlider.getValue()/100;
      if ((m_pPCA != null) && (m_pPCA.getRobustnessParameter() != fRobustness)) {
        calculatePCA(true);
      }
    }
    catch(Exception ex) {
      m_pRobustnessTextField.setText(m_pRobustnessSlider.getValue() + " %");
    }

  }


  ////////////////////////////////////////
  //CHANGELISTENER METHOD               //
  ////////////////////////////////////////
  //robustness slider has changed
  public void stateChanged(ChangeEvent e) {
    m_pRobustnessTextField.setText(m_pRobustnessSlider.getValue() + " %");

    //to do: this construct is often found -> give it into a method
    float fRobustness = (float)m_pRobustnessSlider.getValue()/100;
    if ((m_pPCA != null) && (m_pPCA.getRobustnessParameter() != fRobustness)) {
      calculatePCA(true);
    }
  }

  ////////////////////////////////////
  // LISTSELECTIONLISTENER          //
  ////////////////////////////////////
  public void valueChanged(ListSelectionEvent e) {
    //1. get the selcted indices
    int[] pSelectedIndices = m_pDimensionsList.getSelectedIndices();
    int iSelectedIndicesNum = pSelectedIndices.length;

    //2. set entries in used PCs choice
    int iChoiceSelection = m_pUsedPCsChoice.getSelectedIndex();
    m_pUsedPCsChoice.removeAll();
    if (iSelectedIndicesNum > 0) {
      for (int i = 0; i < iSelectedIndicesNum; i++) {
        m_pUsedPCsChoice.add((i+1)+"");
      }

      if (iChoiceSelection == -1) {
        m_pUsedPCsChoice.select(0);
      }
      else {
        if (iChoiceSelection > m_pUsedPCsChoice.getItemCount()-1) {
          iChoiceSelection = m_pUsedPCsChoice.getItemCount()-1;
        }
        m_pUsedPCsChoice.select(iChoiceSelection);

      }
    }


    //3. check if we have to recalculate the pca
    if (iSelectedIndicesNum == 1) {
      m_pExplainedVarianceLabel.setText("Explained variance: 100 %");
      return;
    }

    if (m_pPCA == null) {
      calculatePCA(true);
    }
    else {
      if (m_pPCA.getPrincipalComponentNumber() != iSelectedIndicesNum) {
        calculatePCA(true);
      }
    }





  }

  ////////////////////////////////////
  // ITEMSTATELISTENER              //
  ////////////////////////////////////
  public void itemStateChanged(ItemEvent e) {
    calculatePCA(false);
  }


  ////////////////////////////////////////
  //WindowsListener methods             //
  ////////////////////////////////////////
  public void windowClosing(WindowEvent event1) {
    setVisible(false);
  }
  public void windowGainedFocus(WindowEvent event2) {}
  public void windowClosed(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}




  ////////////////////////////////////
  // PCA FUNCTIONALITY              //
  ////////////////////////////////////

  //calculates the pca if needed and updates the explained variance label
  private void calculatePCA(boolean bNewPCA) {

    //TO DO: check if we have to calculate a new PCA

    //1. calculate a new PCA if necessary
    if ((bNewPCA) || (m_pPCA == null)) {

      System.out.println("recalculate pca");

      //1.1. robustness parameter h
      float fRobustness;
      if (m_pClassicRadioButton.isSelected()) {
        fRobustness = 1.0f;
      }
      else {
        fRobustness = (float)m_pRobustnessSlider.getValue()/100.0f;
      }

      //1.2. create a new data array for the pca
      Object[] pDimensions = m_pDimensionsList.getSelectedValues();
      int iDimNum = pDimensions.length;
      int iEntryNum = m_pData.getEntryNum();
      float[][] pDataArray = new float[iDimNum][iEntryNum];

      for (int iDim = 0; iDim < iDimNum; iDim++) {
        String pDimName = (String)pDimensions[iDim];
        pDataArray[iDim] = m_pData.getDimensionValues(pDimName);
      }

      //1.3. calculate pca
      m_pPCA = new PrincipalComponentAnalysis(pDataArray, fRobustness);
    }


    //2. get the explained variance
    int[] pIndices = new int[m_pUsedPCsChoice.getSelectedIndex() + 1];
    int iIndicesNum = pIndices.length;
    for (int i = 0; i < iIndicesNum; i++) {
      pIndices[i] = i+1;
    }
    float fExplainedVariance = m_pPCA.getExplainedVarianceByComponents(pIndices);
    fExplainedVariance = (float)Math.round(fExplainedVariance*1000)/10;


    m_pExplainedVarianceLabel.setText("Explained variance: " + fExplainedVariance + " %");

  }


  //adds a PCADataDimension to the data
  private void savePCAtoData() {

    //1. check if at least one dimension has been selected
    if (m_pDimensionsList.getSelectedIndices().length <= 1) {
      String pTitle = "Error";
      String pMessage = "At least 2 dimensions for creating the PCA have to be selected.";
      JOptionPane.showMessageDialog(this, pMessage, pTitle, JOptionPane.ERROR_MESSAGE);
      return;
    }

    
    //2. check if the PCA has been calculated
    if (m_pPCA == null) {
      String pTitle = "Error";
      String pMessage = "No dimension for creating the PCA is selected.";
      JOptionPane.showMessageDialog(this, pMessage, pTitle, JOptionPane.ERROR_MESSAGE);
      return;
    }


    //3. check if a name for the principal component dimension has been specified
    if (m_pSaveNameTextField.getText().equals("")) {
      String pTitle = "Error";
      String pMessage = "No name for the principal component dimensions is specified.";
      JOptionPane.showMessageDialog(this, pMessage, pTitle, JOptionPane.ERROR_MESSAGE);
      return;
    }


    //4. fetch the dimension names
    Vector<String> pDimNames = new Vector<String>();
    Object[] pDimensions = m_pDimensionsList.getSelectedValues();
    int iDimNum = pDimensions.length;
    for (int i = 0; i < iDimNum; i++) {
      pDimNames.add((String)pDimensions[i]);
    }

    //5. add the principal components to the data
    int[] pPCAIndices = new int[m_pUsedPCsChoice.getSelectedIndex() + 1];
    int iPCADimNum = pPCAIndices.length;
    for (int i = 0; i < iPCADimNum; i++) {
      float[] pValues = m_pPCA.getComponentCoordinatesForData(i+1);
      ValueAndVector pPrincipalComponents = m_pPCA.getPrincipalComponent(i+1);
      PCADataDimension pPCADimension = new PCADataDimension(pValues, m_pSaveNameTextField.getText() +" " + (i+1), pDimNames, pPrincipalComponents, (i+1));
      m_pData.addDimension(pPCADimension);
    }

    //6. print success message
    String pTitle = "PCA saved.";
    String pMessage = "The principal component dimensions have been added to the dataset.";
    JOptionPane.showMessageDialog(this, pMessage, pTitle, JOptionPane.INFORMATION_MESSAGE);



  }


}