package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.TimerTask;

public class RedrawTask extends TimerTask {

  /*** member variables ***/
  private ParCoordView m_pParCoordView;
  private ScatterplotView m_pScatterplotView;
  private OutlierDetectionView m_pOutlierDetectionView;

  ///////////////////////////////
  // CONSTRUCTORS              //
  ///////////////////////////////
  public RedrawTask(ParCoordView pParCoordView) {
    m_pParCoordView = pParCoordView;
    m_pScatterplotView = null;
    m_pOutlierDetectionView = null;
  }

  public RedrawTask(ScatterplotView pScatterplotView) {
    m_pParCoordView = null;
    m_pScatterplotView = pScatterplotView;
    m_pOutlierDetectionView = null;
  }


  public RedrawTask(OutlierDetectionView pOutlierDetectionView) {
    m_pParCoordView = null;
    m_pScatterplotView = null;
    m_pOutlierDetectionView = pOutlierDetectionView;
  }


  /////////////////////////////////
  // TIMER TASK FUNCTIONALITY    //
  /////////////////////////////////
  public void run() {
    if (m_pParCoordView != null) {
      m_pParCoordView.repaintData();
    }
    if (m_pScatterplotView != null) {
      m_pScatterplotView.repaintData();
    }
    if (m_pOutlierDetectionView != null) {
      m_pOutlierDetectionView.repaintData();
    }
  }


}