package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import data.Data;
import data.DataDimension;
import listeners.GlobalConstants;
import tools.CovarianceCalculator;
import tools.DistanceCalculator;

import javax.swing.JFrame;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.TitledBorder;


//frame for controlling the calculation of the robust and the mahalanobis distance
public class RobustDistanceFrame extends JFrame implements WindowListener, ActionListener,
ChangeListener, ListSelectionListener, GlobalConstants {

  //member variables

  /*** GUI elements  ***/
  private JSlider m_pRobustnessSlider;
  private JTextField m_pRobustnessTextField;
  private JButton m_pSaveButton;

  /*** data elements ***/
  private Data                    m_pData;                 //data object
  private JList                   m_pDimensionsList;         //list holding the relevant dimensions for PCA
  private OutlierDetectionFrame   m_pOutlierDetectionFrame;  //frame handling the outlier detection


 //Constructor, Creating of GUI
  public RobustDistanceFrame(Data pData, OutlierDetectionFrame pOutlierDetectionFrame) {

    //1. member variables
    m_pData = pData;
    m_pOutlierDetectionFrame = pOutlierDetectionFrame;


    //2. Window-Properties
    setResizable(true);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    setSize(new Dimension(500, 400));
    setTitle("Outlier Detection: Robust Distance");
    addWindowListener(this); //add Listener functionality
    


    //3. Containers-Properties
    Container pContainer = this.getContentPane();
    GridBagLayout pGridBagLayout = new GridBagLayout();
    GridBagConstraints pConstraints = new GridBagConstraints();
    pContainer.setLayout(pGridBagLayout);


    //4. creating the panels
    JPanel pRobustParamsPanel = createRobustParamsPanel();
    JPanel pDistanceParamsPanel = createDistanceParamsPanel();
    JPanel pButtonsPanel = createButtonPanel();


    //5. add components to container
    pConstraints.fill = GridBagConstraints.BOTH;
    pConstraints.weightx = 1.0;
    pConstraints.ipadx = 50;
    pConstraints.insets = new Insets(5, 5, 5, 5);

    pConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagLayout.setConstraints(pRobustParamsPanel, pConstraints);
    pContainer.add(pRobustParamsPanel);

    pGridBagLayout.setConstraints(pDistanceParamsPanel, pConstraints);
    pContainer.add(pDistanceParamsPanel);

    pGridBagLayout.setConstraints(pButtonsPanel, pConstraints);
    pContainer.add(pButtonsPanel);

  }


  ////////////////////////////////////
  // GUI CREATION METHODS           //
  ////////////////////////////////////



  private JPanel createRobustParamsPanel() {
    //1. create the panel and the layout
    JPanel pRobustParamsPanel = new JPanel();
    GridBagLayout pGridBagLayout = new GridBagLayout();
    GridBagConstraints pConstraints = new GridBagConstraints();
    pRobustParamsPanel.setLayout(pGridBagLayout);


    //2. create the border
    TitledBorder pRobustParamsBorder = new TitledBorder("Robustness parameter");
    pRobustParamsPanel.setBorder(pRobustParamsBorder);


    //ROBUST PARAMETER H (which percentage of the data)

    //3. labels
    JLabel pRobustnessLabel = new JLabel("Percentage of data (h) for robust estimate");
    JLabel pRobustnessMinLabel = new JLabel("50 %");
    JLabel pRobustnessMaxLabel = new JLabel("100 %");
    pRobustnessMaxLabel.setHorizontalAlignment(JLabel.RIGHT);


    //4. JSlider
    m_pRobustnessSlider = new JSlider(50, 100, 50);
    m_pRobustnessSlider.setEnabled(true);
    m_pRobustnessSlider.addChangeListener(this);

    //5. TextField
    m_pRobustnessTextField = new JTextField("50 %");
    m_pRobustnessTextField.setHorizontalAlignment(JTextField.RIGHT);
    m_pRobustnessTextField.setEnabled(true);
    m_pRobustnessTextField.addActionListener(this);
    m_pRobustnessTextField.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        robustnessTextField_actionPerformed(e);
      }
    });


    //6. add the components to the panel
    pConstraints.fill = GridBagConstraints.BOTH;
    pConstraints.weightx = 1.0;
    pConstraints.ipadx = 50;
    pConstraints.insets = new Insets(5, 5, 5, 5);

    pConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagLayout.setConstraints(pRobustnessLabel, pConstraints);
    pRobustParamsPanel.add(pRobustnessLabel);

    pConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagLayout.setConstraints(m_pRobustnessSlider, pConstraints);
    pRobustParamsPanel.add(m_pRobustnessSlider);

    pConstraints.gridwidth = 1;
    pConstraints.weightx = 1;
    pGridBagLayout.setConstraints(pRobustnessMinLabel, pConstraints);
    pRobustParamsPanel.add(pRobustnessMinLabel);

    pConstraints.weightx = 0;
    pGridBagLayout.setConstraints(m_pRobustnessTextField, pConstraints);
    pRobustParamsPanel.add(m_pRobustnessTextField);


    pConstraints.weightx = 1;
    pConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagLayout.setConstraints(pRobustnessMaxLabel, pConstraints);
    pRobustParamsPanel.add(pRobustnessMaxLabel);

    return pRobustParamsPanel;
  }



  private JPanel createDistanceParamsPanel() {
    //TO DO: those panels with the dimension list are used often. There should be a separate panel class for that.
    //1. create the panel and the layout
    JPanel pDistanceParamsPanel = new JPanel();
    pDistanceParamsPanel.setLayout(new FlowLayout());

    //2. create the border
    TitledBorder pDistanceParamsBorder = new TitledBorder("PCA parameters");
    pDistanceParamsPanel.setBorder(pDistanceParamsBorder);


    //3. Create a JList that displays the dimensions
    String[] pDimNames = m_pData.getDimensionNames();
    int iDimNum = pDimNames.length;

    //delete DOI from list if the list contains it
    for (int i = 0; i < iDimNum; i++) {
      if (pDimNames[i].equals(DOINAME)) {
        String[] pNewDimNames = new String[iDimNum-1];
        int addcount = 0;
        for (int j = 0; j < iDimNum-1; j++) {
          if (j == i) {
            addcount = 1;
          }
          pNewDimNames[j] = pDimNames[j+addcount];
        }
        pDimNames = pNewDimNames;
        iDimNum--;
        break;
      }

    }
    m_pDimensionsList = new JList(pDimNames);
    m_pDimensionsList.addListSelectionListener(this);

    JScrollPane pScrollPane = new JScrollPane(m_pDimensionsList);
    pScrollPane.setPreferredSize(new Dimension(200, 100));
    pScrollPane.setMinimumSize(new Dimension(200, 100));
    pScrollPane.setMaximumSize(new Dimension(200, 100));





    //4. add components to the panel and return the panel
    pDistanceParamsPanel.add(pScrollPane);

    return pDistanceParamsPanel;
  }






  private JPanel createButtonPanel() {

    //1. create the panel and the layout
    JPanel pButtonsPanel = new JPanel();
    pButtonsPanel.setLayout(new FlowLayout());

    //2. buttons
    m_pSaveButton = new JButton("Calculate distance");
    m_pSaveButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        finishDistanceCalculation();
      }
    });
    //bSavePCA.addActionListener(this);

    //3. add components to panel and return the panel
    pButtonsPanel.add(m_pSaveButton);

    return pButtonsPanel;
  }

  ////////////////////////////////////////
  //ACTIONLISTENER methods              //
  ////////////////////////////////////////
  public void actionPerformed(ActionEvent e) {
    //System.out.println(e.getActionCommand());



  }

  //action of changing the value in the robustness textfield
  private void robustnessTextField_actionPerformed(ActionEvent e) {
    try {
      int iValue = Integer.parseInt(m_pRobustnessTextField.getText());

      if (iValue < 50) {
        iValue = 50;
        m_pRobustnessTextField.setText("50 %");
      }

      if (iValue > 100) {
        iValue = 100;
        m_pRobustnessTextField.setText("100 %");
      }

      m_pRobustnessSlider.setValue(iValue);
    }
    catch(Exception ex) {
      m_pRobustnessTextField.setText(m_pRobustnessSlider.getValue() + " %");
    }

  }

  ////////////////////////////////////
  // LISTSELECTIONLISTENER          //
  ////////////////////////////////////
  public void valueChanged(ListSelectionEvent e) {
    /*int[] selected = m_pDimensionsList.getSelectedIndices();

    //set entries in used PCs choice
    int choiceSelection = m_chUsedPCs.getSelectedIndex();
    m_chUsedPCs.removeAll();
    if (selected.length > 0) {
      for (int i = 0; i < selected.length; i++) {
        m_chUsedPCs.add((i+1)+"");
      }

      if (choiceSelection == -1) {
        m_chUsedPCs.select(0);
      }
      else {
        if (choiceSelection > m_chUsedPCs.getItemCount()-1) {
          choiceSelection = m_chUsedPCs.getItemCount()-1;
        }
        m_chUsedPCs.select(choiceSelection);


      }
    }


    //check when if we have to recalculate the pca
    if (selected.length == 1) {
      calculatePCA(true);
      return;
    }

    if (m_PCA == null) {
      calculatePCA(true);
    }
    else {
      if (m_PCA.getPrincipalComponentNumber() != selected.length) {
        calculatePCA(true);
      }
    }*/


  }



  ////////////////////////////////////////
  //CHANGELISTENER METHOD               //
  ////////////////////////////////////////
  public void stateChanged(ChangeEvent e) {
    m_pRobustnessTextField.setText(m_pRobustnessSlider.getValue() + " %");
  }


  ////////////////////////////////////////
  //WindowsListener methods             //
  ////////////////////////////////////////
  public void windowClosing(WindowEvent event1) {
    setVisible(false);
  }
  public void windowGainedFocus(WindowEvent event2) {}
  public void windowClosed(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}





  ////////////////////////////////////
  // DISTANCE FUNCTIONALITY         //
  ////////////////////////////////////

  private void finishDistanceCalculation() {


    //1. check errors
    if (m_pDimensionsList.getSelectedIndices().length <= 1) {
      String pTitle = "Error";
      String pMessage = "At least 2 dimensions for calculating the location and spread of the data have to be selected.";
      JOptionPane.showMessageDialog(this, pMessage, pTitle, JOptionPane.ERROR_MESSAGE);
      return;
    }



    //2. create a copy of the data needed
    Object[] pDimensions = m_pDimensionsList.getSelectedValues();
    int iDimNum = pDimensions.length;
    int iEntryNum = m_pData.getEntryNum();

    float[][] pDataArray = new float[iDimNum][iEntryNum];

    for (int iDim = 0; iDim < iDimNum; iDim++) {
      String pDimName = (String)pDimensions[iDim];
      pDataArray[iDim] = m_pData.getDimensionValues(pDimName);
    }



    //3. robust distances
    //3.1.  calculate robust covariance matrix (spread)
    float[][] pRobustCovarianceMatrix;
    CovarianceCalculator pCovarianceCalculator = new CovarianceCalculator();

    float fRobustness = (float)m_pRobustnessSlider.getValue()/100.0f;
    pRobustCovarianceMatrix = pCovarianceCalculator.robustCovarianceMatrix(fRobustness, pDataArray);

    //3.2. get the robust mean (location)
    float[] pRobustMeanVector = pCovarianceCalculator.getMeanVector();


    //3.3. calculate the robust distances
    float[] pRobustDistances = DistanceCalculator.getMahalanobisDistancesAsFloatArray(pDataArray, pRobustCovarianceMatrix, pRobustMeanVector);



    //4. mahalanobis distances
    //4.1. calculate classic covariance matrix (spread)
    float[][] pClassicCovarianceMatrix;
    pClassicCovarianceMatrix = pCovarianceCalculator.classicCovarianceMatrix(pDataArray);

    //4.2. get the classic mean (location)
    float[] pMeanVector = pCovarianceCalculator.getMeanVector();


    //4.3. calculate the mahalanobis distance
    float[] pMahalanobisDistances = DistanceCalculator.getMahalanobisDistancesAsFloatArray(pDataArray, pClassicCovarianceMatrix, pMeanVector);




    //5. set the distances in the outlier detection view
    m_pOutlierDetectionFrame.setDistances(pRobustDistances, pMahalanobisDistances, iDimNum);
    m_pOutlierDetectionFrame.setVisible(true);

    setVisible(false);





  }


}